## begin license ##
#
#    "CQ2 Utils Old" (cq2utils_old) is a package with a wide range of valuable tools for python2.4x.
#    Copyright (C) 2005-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "CQ2 Utils Old".
#
#    "CQ2 Utils Old" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "CQ2 Utils Old" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "CQ2 Utils Old"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
#
# Copyright AMARA
#
from amara.binderytools import InputSource, bindery, DEFAULT_URI, bind_file, bind_string


def bind_stream(stream, uri=None, rules=None, binderobj=None, prefixes=None):
	"""
	Create a binding from XML retrieved from a file-like object
	rules - a list of bindery rule objects to fine-tune the binding
	binderobj - optional binder object to control binding details,
		the default is None, in which case a binder object
		will be created
	prefixes - dictionary mapping prefixes to namespace URIs
		the default is None
	"""
	rules = rules or []
	#Create an input source for the XML
	isrc_factory = InputSource.DefaultFactory
	isrc = isrc_factory.fromStream(stream, uri or DEFAULT_URI)
	
	# According to RFC 3023 the encoding of a stream should be set using
	# the content-type header. "Content-Type: text/xml; charset=utf-8"
	# If this charset is not specified us-ascii is assumed. We override
	# this behaviour by defaulting to utf-8.
	if isrc.encoding == 'us-ascii': 
		isrc.encoding = 'utf-8'

	if binderobj is None:
		binderobj = bindery.binder(prefixes=prefixes)
	for rule in rules:
		binderobj.add_rule(rule)

	#Now bind from the XML given in the input source
	binding = binderobj.read_xml(isrc)
	return binding

def bind_uri(uri, rules=None, binderobj=None, prefixes=None):
	"""
	Create a binding from XML retrieved from a URI
	rules - a list of bindery rule objects to fine-tune the binding
	binderobj - optional binder object to control binding details,
							the default is None, in which case a binder object
							will be created
	prefixes - dictionary mapping prefixes to namespace URIs
							the default is None
	"""
	rules = rules or []
	#Create an input source for the XML
	isrc_factory = InputSource.DefaultFactory
	isrc = isrc_factory.fromUri(uri)
	
	# According to RFC 3023 the encoding of a stream should be set using
	# the content-type header. "Content-Type: text/xml; charset=utf-8"
	# If this charset is not specified us-ascii is assumed. We override
	# this behaviour by defaulting to utf-8.
	if isrc.encoding == 'us-ascii': 
		isrc.encoding = 'utf-8'

	if binderobj is None:
		binderobj = bindery.binder(prefixes=prefixes)
	for rule in rules:
		binderobj.add_rule(rule)

	#Now bind from the XML given in the input source
	binding = binderobj.read_xml(isrc)
	return binding
